{% extends 'allianceauth/base.html' %}

{% load i18n %}

{% block page_title %}
    {% translate "Blueprint Library" %}
{% endblock %}

{% block content %}
    <div class="aa-blueprints">
        <h1 class="page-header text-center">{% translate "Blueprint Library" %}</h1>

        {% block details %}{% endblock %}
    </div>
{% endblock %}
