{% load i18n %}
{% load static %}
{% load humanize %}

<div class="modal-content">
    <div class="modal-header">
        <button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">
            &times;
        </span></button>
        <h4 class="modal-title" id="modalTitle">{% translate 'View Request' %}</h4>
    </div>

    <div class="modal-body">

        <div class="form-horizontal">
            <div class="form-group">
                <label class="col-sm-4">{% translate "Blueprint Type" %}</label>
                <div class='col-sm-8'>{{ request.type_icon }} {{ request.type_name }}</div>
            </div>
            <div class="form-group">
                <label class="col-sm-4">{% translate "Blueprint Location" %}</label>
                <div class='col-sm-8'>{{ request.location }}</div>
            </div>
            <div class="form-group">
                <label class="col-sm-4">{% translate "Material Efficiency" %}</label>
                <div class='col-sm-8'>{{ request.material_efficiency }}</div>
            </div>
            <div class="form-group">
                <label class="col-sm-4">{% translate "Time Efficiency" %}</label>
                <div class='col-sm-8'>{{ request.time_efficiency }}</div>
            </div>
            <div class="form-group">
                <label class="col-sm-4">{% translate "Runs" %}</label>
                <div class='col-sm-8'>
                    {% if request.runs %}
                        {{ request.runs }}
                    {% else %}
                        &mdash;
                    {% endif %}
                </div>
            </div>
        </div>
    </div>

    <div class="modal-footer">
        </div>
</div>
